#
# VFS for ruby - open anything with URI
# 
#   Copyright: Tatsuki Sugiura <sugi@nemui.org>
#   License: Ruby's original or GPL2
#   $LastChangedDate: 2003-11-10 19:36:16 +0900 (Mon, 10 Nov 2003) $
#   $LastChangedBy: sugi $
#

require 'net/ftp'

module VFS;  module SchemeHandler;  module FTP;
  module Common
    def self.cleanup(conn)
      Proc.new {
        unless conn.closed?
          VFS::Log.log("VFS(FTP): closing connection.", Log::INFO)
          conn.quit
        end
      }
    end

    def init_common
      @conn   = Net::FTP.new
      cleaner = VFS::SchemeHandler::FTP::Common.cleanup(@conn)
      ObjectSpace.define_finalizer(self, cleaner)
    end

    def close_socket
      VFS::SchemeHandler::FTP::Common.cleanup(@conn)
    end

    def ftp_connect
      if @conn.closed?
        log("opening connection.", Log::INFO)
        @conn.connect(uri.host, uri.port)
        @conn.login(uri.user ? URI.unescape(uri.user) : "anonymous",
                    uri.password ? URI.unescape(uri.password) : nil)
      end
      @conn
    end

    def ftp_request
      if block_given?
        yield(ftp_connect)
      else
        ftp_connect
      end
    end
    
  end
end; end; end
