#
# VFS for ruby - open anything with URI
# 
#   Copyright: Tatsuki Sugiura <sugi@nemui.org>
#   License: Ruby's original or GPL2
#   $LastChangedDate: 2003-10-22 08:13:22 +0900 (Wed, 22 Oct 2003) $
#   $LastChangedBy: sugi $
#

require 'vfs/compat'
require 'zip/zip'
require 'zip/zipfilesystem'

module VFS;  module SchemeHandler;  module ZIP;
  module Common
    def parse_zipurl(uri)
      m = nil
      if m = Regexp.new("^zip:([^!]+)!(.+)$").match(uri.to_s)
      elsif m = Regexp.new("^zip:(.*?/[^/]+\.(?i:zip))/(.+)$").match(uri.to_s)
      else
        raise Error::InvalidURI,
              "invalid zip URL '#{uri}' was speficied for zip scheme handler"
      end
      [m[1], m[2]]
    end

    def initialize(uri)
      super
      @ziparc, @zipcont = parse_zipurl(uri)
      @zipentry = nil
      log("initialized archive=#{@ziparc}, content=#{@zipcont}", Log::INFO)
    end
    attr_reader :ziparc, :zipcont
    private     :ziparc, :zipcont

    def ziparc_open(create = false)
      VFS::CompatLayer.override
      zf = Zip::ZipFile.open(ziparc, create)
      if block_given?
        begin
          yield(zf)
        ensure
          zf.close
          VFS::CompatLayer.restore
        end
      else
        zf
      end
    end
    private :ziparc_open

    def zipentry
      return @zipentry unless @zipentry == nil
      ziparc_open { |zf|
        @zipentry = zf.find_entry(zipcont)
      }
      unless @zipentry
        # FIXME. correct error
        raise Errno::ENOENT
      end
      @zipentry
    end
    private :zipentry

  end # Common
end; end; end
